#ifndef __el__game_h
#define __el__game_h

#include <iostream>
#include "SDL.h"

#include "container.h"

#include "coordinate.h"
#include "map.h"
#include "particles.h"
#include "timeleft.h"
#include "energy.h"
#include "audio.h"
#include "interval.h"

namespace PRJID {

class Entity;
class Map;


typedef enum E_Action {
	A_NONE,
	A_START,
	A_QUIT,
	
	A_FORWARD,
	A_BACKWARD,
	A_LEFT,
	A_RIGHT,
	A_TURNLEFT,
	A_TURNRIGHT,
	A_TURN,
	A_DAMAGE,
	A_ADDPARTICLE,

	A_LAST
};


typedef enum E_GameState {
	GS_NONE,
	GS_PLAYING,

	GS_FINISHED,
	GS_NEXTLEVEL,

	GS_FAILED,
	GS_GAMEOVER,

	GS_QUIT
};


typedef enum E_Diffi {
	D_EASY=1,
	D_MEDIUM=2,
	D_HARD=4,
	D_NIGHTMARE=10
};


class Game {
	Container container_;
	Entity * player_;
	Map map;
	Particles particles;
	unsigned int ticks_;
	void rotrans();

	unsigned int score;
	int dx,dy;

	unsigned int lastparticle;
	unsigned int level;

	Timeleft timeleft;
	Energy energy;
	Timeleft fin;
	Interval particleinterval;
	Timeleft startmessage;

	E_Diffi difficulty;
	E_GameState state;
	bool drawmap;
public:
	Game(Game *g=NULL,E_Diffi d=D_EASY);
	~Game();

	bool inprogress();
	void addparticle();

	Entity *get_player() { return player_; }
	unsigned int get_score() { return score; }
	unsigned int get_level() { return level; }
	E_Diffi get_difficulty() { return difficulty; }
	E_GameState get_state() { return state; }

	bool isactive();

	int action(E_Action id,int x=0,int y=0);

	void cap();

	bool isvalid(C3f p);
	void tick(unsigned int ticks);

	void draw();
	void handle_state();
	bool handle_event(SDL_Event event);
	bool handle_game_event(SDL_Event event);
};

std::ostream& operator<<(std::ostream&s, Game);

} //namespace

#endif
