
#include "SDL_opengl.h"

#include "map.h"
#include "cube.h"
#include "blockaddress.h"
#include <time.h>


#include "tweak/tweak.h"

namespace PRJID {


void Map::init(std::string name) {
	dl=0;
	selected.reset_pos();

	load(name);
	block.split();
}

static std::string path="";
void Map::set_path(std::string s) {
	path=s;
}

void Map::select(unsigned int x,unsigned int y) {
	Blockaddress a;

	glPushMatrix();
	float scale=Tweak::blockscale_f();
	glScalef(scale,scale,scale);
	a=block.top_select(x,y);
	glPopMatrix();

	selected=a;
}


void Map::load(std::string name) {
	if (!block.top_load(path+name)) {
		std::cout << "error " << std::endl;
	}
}
void Map::save() {
	std::ostringstream str;
	str << "backup/save_" << time(NULL);
	if (!block.top_save("save")) { std::cout << "error " << std::endl; }
	if (!block.top_save(str.str())) { std::cout << "error " << std::endl; }
}


void Map::del_dl() {

	//std::cout << "deleting display list " << dl << std::endl;
	glDeleteLists(dl,1);
	dl=0;
}

void Map::gen_dl() {
	dl = glGenLists(1);
	glNewList(dl,GL_COMPILE);
	glDisable(GL_LIGHTING);
	glColor3f(.0,.0,.0);
	Blockaddress a;
	glPushMatrix();
	float scale=Tweak::blockscale_f();
	glScalef(scale,scale,scale);
	block.get_stat();
	block.draw(a);
	glPopMatrix();
	glEndList();
	//std::cout << "generated display list " << dl << "   stat: " << block.get_stat() << std::endl;
}
void Map::draw() {
	glCallList(dl);
}

void Map::debug_draw(bool color) {
	glEnable(GL_COLOR_MATERIAL);
	if (color) {
		glColor3f(.2,.3,.5);
	} else {
		glColor3f(.0,.0,.0);
	}

	Blockaddress a;

	a.select(selected);

	glPushMatrix();
	float scale=Tweak::blockscale_f();
	glScalef(scale,scale,scale);

	block.draw(a);
	
	glPopMatrix();
	glDisable(GL_COLOR_MATERIAL);
}


bool Map::does_collide(C3f p,float size) {
	return block.top_does_collide(p,size);
}
Blockaddress Map::random() {
	return block.random(Blockaddress());	
}


void Map::check() {
	if (Tweak::editmode_i()!=1) return;
	del_dl();
	block.check();
	gen_dl();
}
void Map::toggle_selected() {
	if (Tweak::editmode_i()!=1) return;
	selected.get_current()->toggle();
}
void Map::split_selected() {
	if (Tweak::editmode_i()!=1) return;
	selected.get_current()->split();
}
void Map::merge_selected() {
	if (Tweak::editmode_i()!=1) return;
	Block *sel;
	sel=selected.get_parent();
	if (!sel) return;
	sel->merge();
}

} //namespace
